/* Double Fault handling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#include "doublefault.h"
#include "ledcycle.h"
#include "backtrace.h"
#include <stdio.h>

extern int _excInBacktrace;

__attribute__((naked,noreturn))
void _excBacktraceRewind();

void _excDoubleFaultCheck(uint32_t lr, const uint32_t * fp,
	const struct _ExcProcessorStatus * stat) {
	int mode = stat->spsr & 0x1f;
	char buf[120];
	
	if (mode == 0x17 || mode == 0x18 || mode == 0x1b) {
		/* double fault detected */
		_excInit();
		
		snprintf(buf, sizeof(buf), "\nDouble Fault Detected: %s Mode -> %s Mode\n",
			stat->spsrModeLabel, stat->cpsrModeLabel);
		_excUartOut(buf);
		
		/* check if the double-fault occured inside a backtracing
		 * if so, that function will turn off name demangling and not return
		 * here but restore the previous state */
		_excBacktraceDoubleFault();
		
		_excPrintProcessorStatus(lr, fp, stat);
		
		_excLedCycle(FALSE);
	}
}

