/* Image Loader
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/

/*
 this is the actual loader.
 it relies on some symbols which are provided by the linkerscript:
  - __data_load_start denotes the load address of the data section
  - __data_start denotes the virtual address of the data section
  - __data_size denotes the size of the data sectiopn
  - __bss_start denotes the virtual address of the bss section
  - __bss_size denotes the size of the bss section
 
 the loader moves the data section from its load address to its virtual address
 and initializes the bss section with 0x0
*/

.section .loader
/* this function loads and initializes all sections.
 * it relies on having a stack which can store 8 words
 */
.globl _loadImage
_loadImage:
.func _loadImage
	/* save lr register */
	mov r4, lr
	
	/* copy data section from its lma to the vma */
	ldr	r0, =__data_start		@ destination
	ldr	r1, =__data_load_start	@ source
	ldr	r2, =__data_size		@ size
	bl memcpy
	
	/* initialize the bss section by zeroing the vma of the section */
	ldr r0, =__bss_start		@ destination
	mov r1, #0x0				@ fill
	ldr r2, =__bss_size			@ size
	bl memset
	
	/* initialize frame pointer */
	mov fp, #0
	
	/* return */
	mov pc, r4
.endfunc

