/* Startup Code for Execute-In-Ram Feature
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/

/* Pre-Initialization when executing the image from RAM
 * first we have to move the whole image at the end of SDRAM
 * then we turn on the MMU to remap that image to the address it expects */
/* \param r0 expected image start
 * \param r1 relative (actual) image start
 */
.section .loader
.globl _startRam
_startRam:
.func _startRam
	/* setup early stack for executing _mmuTableRemap */
	mov sp, r1
	
	/* save r0 and r1 into r4, r5 */
	mov r4, r0
	mov r5, r1
	
	/* modify mmu table:
	 * the mmu table in that image (in SDRAM) is located, modified and loaded */
	/* get the mmu table */
	ldr r0, =_mmuTable	@ absolute address of mmu table
	sub r0, r4			@ subtract image start (=> offset of mmu table in image)
	add r0, r5			@ add relative image start (=> absolute address of table in memory)
	mov r1, r4			@ argument 1: virtual address
	mov r2, r5			@ argument 2: physical address
	ldr r3, =__image_size
	add r3, #4			@ argument 3: length
	bl _mmuTableRemap	@ remap and enable mmu (and caches)
	
	/* now jump into the remapped space :
	 * load absolute address of next '1' label into pc */
	ldr pc, =1f
1:
	/* from here, the remapping is active and we are executing at the expected
	 * address (which is remapped to the physical address the image is
	 * located) */
	
	/* setup stack for all significant modes according to the symbols provided
	 * by the linker script (see stack.s for more information) */
	ldr r0, =_STACK_END
	cmp r0, r5
	movhi r0, r5	@ limit stack to r5 (image start)
	bl _stackSetup
	
	/* load all sections of the image to their respective virtual memory
	 * addresses (see loader.s) */
	bl _loadImage
	
	b _startMain
.endfunc

